clc;
clear;
close all;

CostFunction = @(x) optimize_Kesirli_PID(x);      

nVar = 4;            

VarSize = [1 nVar];   

VarMin = [0.0001 0.0001 0.0001 0.0001];       
VarMax = [1 1 1 1];         


%% ACOR Parameters

MaxIt = 100;         % Maximum Number of Iterations

nPop = 30;             % Population Size (Archive Size)

nSample = 40;         % Sample Size

q = 0.5;              % Intensification Factor (Selection Pressure)

zeta = 1;             % Deviation-Distance Ratio

%% Initialization

% Create Empty Individual Structure
empty_individual.Position = [];
empty_individual.Cost = [];

% Create Population Matrix
pop = repmat(empty_individual, nPop, 1);

% Initialize Population Members
for i = 1:nPop
    
    % Create Random Solution
    pop(i).Position = unifrnd(VarMin, VarMax, VarSize);
    
    % Evaluation
    pop(i).Cost = feval(CostFunction,pop(i).Position);
    
end

% Sort Population
[~, SortOrder] = sort([pop.Cost]);
pop = pop(SortOrder);

% Update Best Solution Ever Found
BestSol = pop(1);

% Array to Hold Best Cost Values
BestCost = zeros(MaxIt, 1);

% Solution Weights
w = 1/(sqrt(2*pi)*q*nPop)*exp(-0.5*(((1:nPop)-1)/(q*nPop)).^2);

% Selection Probabilities
p = w/sum(w);


%% ACOR Main Loop

for it = 1:MaxIt
    
    % Means
    s = zeros(nPop, nVar);
    for l = 1:nPop
        s(l, :) = pop(l).Position;
    end
    
    % Standard Deviations
    sigma = zeros(nPop, nVar);
    for l = 1:nPop
        D = 0;
        for r = 1:nPop
            D = D+abs(s(l, :)-s(r, :));
        end
        sigma(l, :) = zeta*D/(nPop-1);
    end
    
    % Create New Population Array
    newpop = repmat(empty_individual, nSample, 1);
    for t = 1:nSample
        
        % Initialize Position Matrix
        newpop(t).Position = zeros(VarSize);
        
        % Solution Construction
        for i = 1:nVar
            
            % Select Gaussian Kernel
            l = RouletteWheelSelection2(p);
            
            % Generate Gaussian Random Variable
            newpop(t).Position(i) = s(l, i)+sigma(l, i)*randn;
            
        end
        
        % Apply Variable Bounds
        newpop(t).Position = max(newpop(t).Position, VarMin);
        newpop(t).Position = min(newpop(t).Position, VarMax);
        
        % Evaluation
        newpop(t).Cost = feval(CostFunction,newpop(t).Position);
        
    end
    
    % Merge Main Population (Archive) and New Population (Samples)
    pop = [pop
         newpop]; %#ok
     
    % Sort Population
    [~, SortOrder] = sort([pop.Cost]);
    pop = pop(SortOrder);
    
    % Delete Extra Members
    pop = pop(1:nPop);
    
    % Update Best Solution Ever Found
    BestSol = pop(1);
    
    % Store Best Cost
    BestCost(it) = BestSol.Cost;
    
    % Show Iteration Information
    disp(['Iteration ' num2str(it) ': Best Cost = ' num2str(BestCost(it)) ': Best Post = ' num2str(BestSol.Position)]);

end


